/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.handler;

import cn.hutool.json.JSONUtil;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocHandleParam;
import com.abssqr.plat.common.model.domain.plan.PlanAllocFrozen;
import com.abssqr.plat.common.model.domain.plan.PlanAstLog;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanAllocFrozenRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanAllocManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.share.pay.PlanCapitalCreateRequest;
import com.abssqr.plat.core.share.pay.PlanCapitalFinishRequest;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.alibaba.fastjson.JSON;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.util.adapter.AbstractBaseActionHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 分配兑付
 * 两个调用场景：
 * 1. 审批通过后直接发起线上支付 ： 调用 handle 方法， 然后获取到支付单结果后执行 allocTitleEntry
 *
 * 2. 审批通过后修改状态为待支付，有人工手工入账，此时没有支付操作 ，调用 execute 方法执行全部操作
 *
 * @author hanfei
 * @version $Id: PlanAllocHandler.java, v 0.1 2019-07-06 11:46 AM hanfei Exp $ 
 */
@Component
public class PlanAllocHandler extends AbstractBaseActionHandler<PlanAllocHandleParam, PlanCapitalOrder> {
    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private PlanAllocManager        planAllocManager;
    @Autowired
    private PlanAstManager          planAstManager;
    @Autowired
    private PlanAllocFrozenRepo     planAllocFrozenRepo;
    @Autowired
    private OrgAcctDayComponent     orgAcctDayComponent;
    /**
     * 分配支付的发起
     * 1. 生成支付单
     *
     * -- 分配入账（可能存在两个调用来源：1. 审批通过后直接发起支付，2. 审批通过后修改状态为待支付，有人工手工入账，此时没有支付操作
     * 2. 更新分配计划状态为已分配
     * 3. 登记分配计划下，各分配事项对应科目的"付"发生额和流水
     */
    @Override
    public PlanCapitalOrder handle(PlanAllocHandleParam handleParam) {
        // 获取转让会计日
        Day accountDate = orgAcctDayComponent.getTrfAcctDay();
        PlanAlloc alloc = handleParam.getAlloc();
        // 对分配计划加锁
        PlanAlloc lock = planAllocManager.lockByAllocNo(alloc.getAllocNo());
        VarChecker.checkEquals(lock.getStatus(), PlanAllocStatusEnum.UNASSIGNE,
                "计划[{0}]当前分配计划[{1}]不能进行分配，状态为[{2}]", alloc.getPlanNo(), alloc.getAllocNo(), alloc.getStatus());
    
        // 分配兑付操作发起支付创建支付订单 明确时分配兑付类型
        PlanCapitalTransTypeEnum transTypeEnum;
        if (alloc.getPlanType() == PlanTypeEnum.SPV){
            transTypeEnum = PlanCapitalTransTypeEnum.NT_DIST;
        }else{
            transTypeEnum = PlanCapitalTransTypeEnum.DIST;
        }
        PlanCapitalCreateRequest request = new PlanCapitalCreateRequest();
        request.setAccountDate(accountDate.getStandardDate());
        // 使用分配计划 编号作为业务编码
        request.setBizNo(alloc.getAllocNo());
        request.setPlanNo(alloc.getPlanNo());
        //  目标机构 填入 计划编号
        request.setTargetOrgCode(alloc.getPlanNo());
        // 获取分配计划应兑付金额总和
        request.setTransAmt(alloc.getTotalConfirmAllocAmt());
        // 应付金额
        request.setPayableAmt(alloc.getTotalConfirmAllocAmt());
        // 交易类型
        request.setTransType(transTypeEnum);
        request.setMemo(MessageUtil.formatMsg("-日期[{2}]分配兑付", alloc.getPayDate()));
        // 1 发起支付 创建支付订单
        PlanCapitalOrder order = planCapitalOrderManager.createCapitalOrder(request);
        LogUtil.info(LOGGER, "计划[{0}-{1}]创建支付交易单type[{2}]cntPty[{3}]amt[{4}]bizNo[{5}]",
                order.getPlanNo(), order.getAccountDate(), transTypeEnum, order.getCntPtyCode(), order.getTransAmt(), order.getBizNo());
    
        // 2 记录清算在资产水位表中 增加支付在途金额 并将冻结金额解冻
        planAstManager.changePlanAst(accountDate.getStandardDate(), alloc.getPlanNo(), (planAst,planAstLogList) -> {
            // 2.1 分配兑付需要进行解冻记录冻结金额解冻变化
            this.unFrozen(planAstLogList, accountDate.getStandardDate(), planAst, order);
    
            // 2.2 更新计划资产水位表，追加在途的支出金额 并登记日志
            planAstLogList.add(planAstManager.itemChangeStage(order.getAccountDate(), planAst, TraderTypeEnum.INITIATOR,
                    StageChangeTypeEnum.ON_WAY, order.getTransAmt(), transTypeEnum,
                    order.getBizNo(), JSONUtil.toJsonStr(order)));
        });
    
        alloc.setStatus(PlanAllocStatusEnum.PENDING);
        alloc.setAllocAmt(order.getTransAmt());
        // 3 变更分配计划 更新为分配中 记录分配总额
        planAllocManager.updateAllocStatus(alloc);
        return order;
    }
    
    /**
     * 发起创建支付单后
     * 1、更新分配计划状态为已分配
     * 2、登记分配计划下，各分配事项对应科目的"付"发生额和流水
     * @param handleParam
     * @param order
     * @return
     */
    @Override
    public PlanCapitalOrder afterHandle(PlanAllocHandleParam handleParam, PlanCapitalOrder order) {
        // 线下交易，更新资金交易单的状态为成功
        if (handleParam.getOffLine()) {
            // 线下支付默认成功 确认金额为交易金额
            PlanCapitalFinishRequest finishRequest = new PlanCapitalFinishRequest();
            finishRequest.setOutTransNo(handleParam.getOutTransNo());
            finishRequest.setSeqNo(order.getSeqNo());
            finishRequest.setCfrmAmt(order.getTransAmt());
            finishRequest.setRes(TaskStatusEnum.SUCCESS);
            finishRequest.setMemo("分配线下支付");
            // 更新 资金单据 支付结果  、 变更水位 、科目入账
            order = planCapitalOrderManager.finishCapitalOrder(finishRequest);
        }
        return order;
    }
    
    /**
     * 解冻冻结金额 目前只面向 分配兑付
     * @param curDate
     * @param planAst
     * @param capitalOrder
     * @return
     */
    private void unFrozen(List<PlanAstLog> planAstLogList, Date curDate,
                          PlanAst planAst, PlanCapitalOrder capitalOrder) {
        // 查询对应的 分配计划冻结记录 同时获取 对应的锁定金额明细进行解冻
        PlanAllocFrozen allocFrozen = planAllocFrozenRepo.getByAllocNo(planAst.getPlanNo(), capitalOrder.getBizNo());
        VarChecker.checkNotNull(allocFrozen,
                "[{0}]未匹配到对应的冻结记录[{1}]", planAst.getPlanNo(), capitalOrder.getBizNo());
        // 判断水位冻结金额 减去 冻结记录金额 后不能小于零
        VarChecker.checkArgument(!planAst.getFrozenCash().subtract(allocFrozen.getFrozenCash()).isNegtive(),
                "[{0}]计划分配编号[{1}]解冻金额{2} 大于 冻结金额{3}",
                planAst.getPlanNo(), capitalOrder.getBizNo(), allocFrozen.getFrozenCash(), planAst.getFrozenCash());
        // 冻结金额解冻记录
        PlanAstLog planAstLog = PlanAstLog.buildAstLog(curDate, planAst, allocFrozen.getAllocNo(),
                TraderTypeEnum.INITIATOR, capitalOrder.getTransType().getTransCode(), StageChangeTypeEnum.UNFROZEN);
        planAstLog.changeFrozenCash(planAst, allocFrozen.getFrozenCash(), capitalOrder.getTransAmt(), JSON.toJSONString(allocFrozen));
        planAstLogList.add(planAstLog);
        // 同时删除冻结金额记录
        planAllocFrozenRepo.deleteByAllocNo(planAst.getPlanNo(), capitalOrder.getBizNo());
    }
}
