/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.handler;

import com.abssqr.plat.common.dal.base.StatEntity;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanClearingLogStat;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.OrgAstStatStatusEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.trans.CapitalTrader;
import com.abssqr.plat.common.facade.model.trans.ChangeStageParam;
import com.abssqr.plat.common.facade.model.trans.PlanClearingOrder;
import com.abssqr.plat.common.model.convertor.PlanClearingOrderConvert;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingLogRepo;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 资产清算统计业务逻辑
 *
 一 修改内容
 1、更改PlanAstStatJob为每日固定时段执行[0 0/5 17-18 * * ? *]\n
 2、增加结算单表 PLAN_NO,TRANS_CODE,ACCOUNT_DATE,CNT_PTY_CODE,TRANS_AMT,TRANS_STATUS
 TRANS_STATUS: TaskStatusEnum.INIT, EXECUTING, WAITING, SUCCESS
 3、水位表增加待清算收入、待清算支出字段
 4、修改水位表中获取可用余额的方法，可用余额= 余额Cash - 冻结金额Frozen - 付在途 - 待结算支出 + 待结算收入
 
 二 PlanAstClearingStatHandler 业务
 1、获取清算统计前置条件数据（最小id，最大id，总数量）;
 2、拉取清算汇总数据按PLAN_NO, TRANS_CODE, ACCOUNT_DATE, CNT_PTY_CODE分组;
 3、按照汇总结果生成对应结算单1-1;
 4、追加水位表中 待结算支出 与 待结算收入 的金额;
 
 三 增加清算支付处理任务
 1、拉取待支付的 结算单TaskStatusEnum.INIT，更新为EXECUTING
 2、根据对手机构进行汇总支付与收入，进行轧差计算，创建支付订单，一个对手机构对应一笔支付单
 PlanCapitalTransTypeEnum 交易类型增加 支付 与 收入 类型进行对应
 3、修改结算单状态为 WAITING，等待支付结果
 
 四 增加交易单结果完成后后续处理
 1、增加选择处理器，根据不同 PlanCapitalTransTypeEnum 类型，进行对应业务处理剥离 分配兑付与清算
 2、清算类型交易单结果处理：
 2.1、拉取交易单据对应交易对手的结算单，根据实际确认金额，循环每条扣减金额及更改状态为SUCCESS
 2.2、对支付金额不足剩余的 结算单 更改状态为 INIT，等待下次支付
 2.3、根据交易单据金额更改扣减水位表的 待结算支出 或 待结算收入 金额
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.handler: PlanAstStatHandler.java, v 0.1 2019-06-12 20:44 zhenxuan.luo Exp $
 */
@Component
public class PlanAstClearingStatHandler {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanAstClearingStatHandler.class);
    
    @Autowired
    private PlanClearingLogRepo   planClearingLogRepo;
    @Autowired
    private PlanAstStatTaskRepo   planAstStatTaskRepo;
    @Autowired
    private PlanClearingOrderRepo planClearingOrderRepo;
    @Autowired
    private PlanAstManager        planAstManager;
    @Autowired
    private TransactionTemplate   transactionTemplate;
    @Autowired
    private PlanRepo              planRepo;
    
    /**
     * 清算流水统计 为交易准备 生成结算单
     * @param planAstStatTask
     * @param execTms
     * @return
     */
    public PlanAstStatTask execute(PlanAstStatTask planAstStatTask, String execTms) {
        planAstStatTask.setLastExecTms(execTms);
        // 获取清算统计前置条件数据 每 20000 笔统计一次
        StatEntity statEntity = planClearingLogRepo.getStatInx(
                planAstStatTask.getAccountDate(), planAstStatTask.getPlanNo(), execTms);
        
        if (statEntity == null || statEntity.getCnt() == 0) {
            LogUtil.warn(LOGGER, "[{0}]计划清算统计任务[{1}]没有数据execTms={2}",
                    planAstStatTask.getPlanNo(), planAstStatTask.getAccountDate(), execTms);
            planAstStatTask.setStatus(OrgAstStatStatusEnum.WAIT);
            planAstStatTaskRepo.updateTask(planAstStatTask);
            return planAstStatTask;
        }
        LogUtil.info(LOGGER, "[{0}]计划清算统计任务[{1}]开始[{2}]条execTms={3}",
                planAstStatTask.getPlanNo(), planAstStatTask.getAccountDate(), statEntity.getCnt(), execTms);
        // 设置统计数据范围
        planAstStatTask.setMinId(statEntity.getMinId());
        planAstStatTask.setMaxId(statEntity.getMaxId());
        planAstStatTask.setCnt(statEntity.getCnt());
        
        // 拉取清算汇总数据 按 PLAN_NO, TRANS_CODE, ACCOUNT_DATE, CNT_PTY_CODE 分组
        List<LoanClearingLogStat> statReses = planClearingLogRepo.clearingLogStat(planAstStatTask.getAccountDate(),
                planAstStatTask.getMaxId(),
                planAstStatTask.getMinId(),
                planAstStatTask.getPlanNo(),
                planAstStatTask.getLastExecTms());
        
        long total = 0;
        for (LoanClearingLogStat stat : statReses) {
            total = total + stat.getCnt();
        }
        // 并发处理判断
        if (total != planAstStatTask.getCnt()) {
            LogUtil.warn(LOGGER, "计划统计任务[{0}]汇总条数和分事件汇总条数[{1}]不一致，重新调度", planAstStatTask, total);
            planAstStatTask.setStatus(OrgAstStatStatusEnum.WAIT);
            planAstStatTaskRepo.updateTask(planAstStatTask);
            return planAstStatTask;
        }
        // 处理统计
        planAstStatTask = this.statHandle(statReses, planAstStatTask, execTms);
        LogUtil.info(LOGGER, "计划统计任务[{0}]结束, execTms={1}", planAstStatTask, execTms);
        return planAstStatTask;
    }
    
    /**
     * 更新水位 创建结算单 ClearingOrder
     * @param statReses
     * @param planAstStatTask
     * @param execTms
     * @return
     */
    private PlanAstStatTask statHandle(final List<LoanClearingLogStat> statReses,
                                       final PlanAstStatTask planAstStatTask, String execTms) {
        Map<String, PlanBase> cntPtyMap = new HashMap<>();
        
        return transactionTemplate.execute(status -> {
            // 对统计任务加锁避免并发
            PlanAstStatTask lockedTask = planAstStatTaskRepo.lock(planAstStatTask.getPlanNo());
            if (lockedTask.getStatus() != OrgAstStatStatusEnum.PROCESS) {
                LogUtil.warn(LOGGER, "计划统计任务[{0}]状态不为处理中，可能存在并发问题", lockedTask);
                return lockedTask;
            }
            
            if (lockedTask.getMinId() != null && lockedTask.getMinId() >= planAstStatTask.getMinId()) {
                throw new AbssqrBizException(
                        MessageUtil.formatMsg("计划统计任务[{0}]最小ID大于待处理的最小ID[{1}]，可能存在并发问题"
                                , lockedTask, planAstStatTask),
                        CommonErrorCodeEnum.INVALID_STATUS, true);
            }
            // locked对象只判断幂等状态不做业务使用
            lockedTask = null;
            
            // 更新水位金额变化 及登记流水
            planAstManager.changePlanAst(planAstStatTask.getAccountDate(), planAstStatTask.getPlanNo(), (planAst, planAstLogList) -> {
                // 记录清算在资产水位表中 待结算的收入支出金额变化
                for (LoanClearingLogStat statRes : statReses) {
                    LogUtil.info(LOGGER, "计划统计结果：planNo={0}, accountDate={1} | statTask={2} | statRes={3}",
                            planAstStatTask.getPlanNo(), planAstStatTask.getAccountDate(), planAstStatTask, statRes);
                    // 汇总金额
                    if (statRes.getAmt().isGreaterThanZero()) {
                        // 1. 创建结算单
                        PlanClearingOrder clearingOrder = PlanClearingOrderConvert.convert2Domain(statRes, execTms);
                        planClearingOrderRepo.createOrder(clearingOrder);
                        LogUtil.info(LOGGER, "计划资产清算汇总结算单：【{0}】---【{1}】",
                                clearingOrder, planAstStatTask);
                        // 2. 备份ClearingLog 将ClearingOrder的 seqNo 回填
                        planClearingLogRepo.backStated(
                                clearingOrder.getSeqNo(), planAstStatTask.getLastExecTms(), statRes);
                        
                        // 查询对手机构
                        PlanBase cntPtyPlan = planRepo.getPlanByMapCache(cntPtyMap, statRes.getCntPtyCode());
                        // 构建交易双方及交易编码
                        CapitalTrader capitalTrader = new CapitalTrader(planAst.getPlanNo(), planAst.getPlanType(),
                                cntPtyPlan.getPlanNo(), cntPtyPlan.getPlanType(), clearingOrder.getTransCode());
                        planAstLogList.addAll(planAstManager.execChangeStageTwoWay(planAst,
                                ChangeStageParam.createParamByClearing(capitalTrader, StageChangeTypeEnum.CLEAR, clearingOrder)));
                    }
                }
            });
            // 4. 删除清算流水
            int refCnt = planClearingLogRepo.deleteStated(
                    planAstStatTask.getAccountDate(), planAstStatTask.getMaxId(),
                    planAstStatTask.getMinId(), planAstStatTask.getPlanNo(), planAstStatTask.getLastExecTms());
            if (refCnt != planAstStatTask.getCnt()) {
                throw new AbssqrBizException(
                        MessageUtil.formatMsg("计划统计任务统计流水变更[{0}]和任务删除数[{1}]不一致", planAstStatTask, refCnt),
                        CommonErrorCodeEnum.INVALID_STATUS, true);
            }
            LogUtil.info(LOGGER, "计划统计任务[{0}]结束, execTms={1}，汇总流水数[{2}]", planAstStatTask, execTms, refCnt);
            
            // 5. 更新任务状态，等待下一次调度
            planAstStatTask.setStatus(OrgAstStatStatusEnum.WAIT);
            planAstStatTaskRepo.updateTask(planAstStatTask);
            return planAstStatTask;
        });
    }
    
}