/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.handler;

import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.model.domain.plan.PlanAstLog;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.AstSyncTaskComponent;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.notice.AbssqrNoticeManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.*;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.Optional;

/**
 * 资金到账情况
 * @author hanfei
 * @version $Id: PlanFundHandler.java, v 0.1 2019-09-10 3:28 PM hanfei Exp $
 */
@Component
public class PlanFundHandler {
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanFundHandler.class);
    @Autowired
    private WkPayManager            wkPayManager;
    @Autowired
    private PlanAstManager          planAstManager;
    @Autowired
    private AcctManager             acctManager;
    @Autowired
    private TransactionTemplate     transactionTemplate;
    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private AbssqrNoticeManager     abssqrNoticeManager;
    @Autowired
    private AstSyncTaskComponent    astSyncTaskComponent;
     @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;

    /**
     * 资金到账情况 执行余额查询
     * 1、查询当前 状态为 存续中 放款池计划 与 承接池计划
     * 2、获取计划对应资金账户
     * 3、查询账户余额
     * 4、更新资金账户余额
     * 是init初始： "AST1015", "投入资本" ，"add", "水位增加"。修改状态为success
     * 是success非初始：计算银行余额 与 账户余额 差值，
     * 多余的进行增加水位余额，条目为 "AST1013", "资金调整增加"，"add", "水位增加"
     * 计算少于的扣减水位余额，条目为 "AST1014", "资金调整减少"，"sub", "水位扣减"
     * @param entity
     * @param curDate
     * @throws JobExecutionException
     */
    public void executeQueryBalance(PlanBase entity, Day curDate) {
        // 校验是否需有未完成支付单
        if (planCapitalOrderManager.checkExistCapital(entity.getPlanNo())) {
            return;
        }
        // 对于 放款池计划需要校验 当日资产同步是否已经完成 账务资金是否对账完成 避免 还款提前入银行账，导致水位加错
        if (astSyncTaskComponent.cantBankBalance(entity, curDate)) {
            return;
        }
        // 1、获取计划对应资金账户 平台虚拟户
        List<Account> accounts = acctManager.getAcctByPlanNoAndType(
                entity.getPlanNo(), AccountTypeEnum.PLAT_VIRTUAL,false);
        AccountAttributeEnum attr;
        if (entity.getPlanType() == PlanTypeEnum.SPV) {
            // 放款池计划 查询回款户
            attr = AccountAttributeEnum.RECV;
        } else {
            // 承接池计划 查询一般业务户
            attr = AccountAttributeEnum.NORMAL;
        }
        // 过滤出对应查询余额的账户
        Optional<Account> platAcctOpt = accounts.stream()
                                                .filter(item -> item.getActAttr() == attr)
                                                .findFirst();
        VarChecker.checkArgument(platAcctOpt.isPresent(),
                "{0}[{1}]找不到[{2}]对应平台虚拟户", entity.getPlanType().getDesc(), entity.getPlanNo(), attr.getDesc());

        Account planAcct = platAcctOpt.get();
        Money bankBal = new Money();
        try {
            // 3、调用外部接口查询账户余额
            WkChannelNoEnum channelNo = WkChannelNoEnum.getPlatChannelNo(planAcct.getChannelNo());
            bankBal.addTo(wkPayManager.balance(planAcct.getActNo(), channelNo));
            LogUtil.info(LOGGER, "{0}[{1}]查询账户余额: {2}",
                    entity.getPlanType().getDesc(), entity.getPlanNo(), bankBal.getAmount());
        } catch (Throwable e) {
            LogUtil.error(LOGGER, e, "[{0}]-ChannelNo[{1}]-ActNo[{2}]调用五矿查询账户余额异常",
                    entity.getPlanNo(), planAcct.getChannelNo(), planAcct.getActNo());
            throw e;
        }
        final String entityNo = entity.getPlanNo();
        transactionTemplate.execute(status -> {
            // 4、变更水位表
            planAstManager.changePlanAst(curDate.getStandardDate(), entityNo, (planAst, planAstLogList) -> {
                if (planAst.getStatus() == TaskStatusEnum.INIT) {
                    if (bankBal.isZero()) {
                        LogUtil.info(LOGGER, "{0}[{1}]查询银行账户余额: 余额为0，不进行处理",
                                planAst.getPlanType().getDesc(), planAst.getPlanNo());
                        return;
                    }
                    // 账户余额初始化
                    PlanAstLog planAstLog = PlanAstLog.buildAstLog(curDate.getStandardDate(), planAst, entityNo,
                            TraderTypeEnum.INITIATOR, TransCodeEnum.CAP_INVEST, StageChangeTypeEnum.ADD);
                    // 资本增加
                    planAstLog.bankSyncChangeStageAndLog(planAst, bankBal, bankBal, "投入资本到账");
                    planAstLogList.add(planAstLog);
                    // 完成初始化 更新状态
                    planAst.setStatus(TaskStatusEnum.SUCCESS);

                } else {
                    // 判断在途金额非0时不查询
                    if (planAst.getRepoOnWay().isGreaterThanZero()
                        || planAst.getPayOnWay().isGreaterThanZero()) {
                        LogUtil.error(LOGGER, "{0}[{1}]账户水位在途金额不为0: [{2}], [{3}]",
                                planAst.getPlanType().getDesc(), planAst.getPlanNo(), planAst.getRepoOnWay(),
                                planAst.getPayOnWay());
                        return;
                    }
                    /**
                     * 判断系统时间与获取会计日 是否是同一天，非同一天不进行查询账户余额
                     * 非使用系统时间下不进行余额校验查询
                     * 针对放款池会在curDate+1日将还款提前打入账户，防止重复加水位
                     */
                    if (!curDate.equalSysDate()) {
                        return;
                    }

                    // 计算 银行账户余额 与 系统账户余额 差值
                    Money diffAmt = bankBal.subtract(planAst.getCash());
                    // 无金额变化
                    if (diffAmt.isZero()) {
                        LogUtil.info(LOGGER, "{0}[{1}]账户余额核对: 银行余额[{2}], 本地余额[{3}]，差异金额为0",
                                planAst.getPlanType().getDesc(), planAst.getPlanNo(), bankBal, planAst.getCash());
                        return;
                    }

                    TransCodeEnum transCode;
                    if (diffAmt.isGreaterThanZero()) {
                        // 资金增加
                        transCode = TransCodeEnum.CAP_ADJ_ADD;
                    } else {
                        // 资金减少 改为 不进行变动 进行告警通知 银行账户余额 与水位余额 存在短账
                        String content = MessageUtil.formatMsg(
                                "{0}[{1}]账户余额核对短帐: 银行余额[{2}], 本地余额[{3}]，差异金额为[{4}]，请排查银行账户短帐具体原因！",
                                planAst.getPlanType().getDesc(), planAst.getPlanNo(), bankBal, planAst.getCash(),
                                diffAmt);
                        abssqrNoticeManager.sendBusinessNotice(entity, planAcct.getActNo(),
                                AbssqrNoticeTypeEnum.BankFundBal, content,
                                abssqrNoticeManager.getAdminReceivers());
                        return;
                    }
                    LogUtil.info(LOGGER, "{0}[{1}]账户余额核对: 银行余额[{2}], 本地余额[{3}]，差异金额为[{4}]",
                            planAst.getPlanType().getDesc(), planAst.getPlanNo(), bankBal, planAst.getCash(), diffAmt);
                    PlanAstLog planAstLog = PlanAstLog.buildAstLog(curDate.getStandardDate(), planAst, entityNo,
                            TraderTypeEnum.INITIATOR, transCode, StageChangeTypeEnum.BANK_BAL_CHECK);
                    planAstLog.bankSyncChangeStageAndLog(planAst, diffAmt, bankBal, "每日账户余额核对资金变化");
                    planAstLogList.add(planAstLog);
                }
            });
            return null;
        });
    }

}
