/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.report;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.MonthReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingLogRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.plan.PlanSyncTaskCtrManager;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.tuple.MutablePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * @author hanfei
 * @version $Id: WkPlanReportChecker.java, v 0.1 2019-12-30 10:29 PM hanfei Exp $ 
 */
@Component
public class WkPlanReportChecker {
    
    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    private PlanSyncTaskCtrManager  planSyncTaskCtrManager;
    @Autowired
    private PlanManager             planManager;
    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private PlanClearingLogRepo     planClearingLogRepo;
    @Autowired
    private PlanReportRepo          planReportRepo;
    
    /**
     * 日报生成校验，报表日：
     * 1、数据同步任务
     * 2、金融服务费统计
     * 3、结算单或支付单
     * 4、清算流水
     * 5、放款池，判断承接池报表生成
     * @param accountDate  机构会计日
     * @param planBase
     * @param planReport
     * @param reportDateField
     * @param planNos
     * @return
     */
    public MutablePair<Boolean, String> checkDailyAstReport(Date accountDate, PlanBase planBase,
                                                            PlanReport planReport,
                                                            DailyReportDateField reportDateField,
                                                            List<String> planNos){
        MutablePair<Boolean, String> rptResultPair = new MutablePair<>(true, "");
        if (null != planReport) {
            // 最近日期报表日
            reportDateField.setRptDate(planReport.getRptDate().getAccountDate());
            // 最近日期报表日 大于 机构会计日 不生成
            if (DateTimeUtil.calcDayDiff(reportDateField.getAccountDate(), accountDate) >= 0) {
                LogUtil.info(LOGGER, "[ReportJob]-[{0}]计划在[{1}]日报表已经生成",
                        planBase.getPlanNo(), reportDateField.desc());
                return rptResultPair;
            }
            // 设置下一天
            reportDateField.toNext();
        } else {
            // 没有报表 直接设置机构会计日
            reportDateField.parse(DateTimeUtil.formatYMD(accountDate));
        }
    
        // 判断当前机构会计日 数据同步，校验数据同步和模型转换是否完成
        Date orgDate = DateTimeUtil.getStandardDate(reportDateField.getAccountDate());
        if (!planSyncTaskCtrManager.checkSyncFinishGtIsTrue(planBase, orgDate, SyncTaskTypeEnum.AST_MOD)) {
            rptResultPair.setRight(MessageUtil.formatMsg("[ReportJob]-[{0}]计划在[{1}]的数据同步任务未完成",
                    planBase.getPlanNo(), orgDate));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
        // 判断当前机构会计日 金融服务费
        if (!planSyncTaskCtrManager.checkSyncFinishGtIsTrue(planBase, orgDate, SyncTaskTypeEnum.AST_SER_FEE)) {
            rptResultPair.setRight(MessageUtil.formatMsg("[ReportJob]-[{0}]计划在[{1}]的金融服务费统计未完成",
                    planBase.getPlanNo(), orgDate));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
        // 查询计划当前未完成结算单及交易支付单的数量
        int unfinishCapitalCnt = planCapitalOrderManager
                                         .reportCheckCapitals(planBase.getPlanNo(), new Day(reportDateField.getAccountDate()));
        if (unfinishCapitalCnt > 0) {
            rptResultPair.setRight(MessageUtil.formatMsg("[ReportJob]-[{0}]计划在[{1}]的资金处理（结算单或支付单）有[{2}]条未处理完毕",
                    planBase.getPlanNo(), reportDateField.getAccountDate(), unfinishCapitalCnt));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
    
        Date unfinishClearingLogDate = planClearingLogRepo.getMinUnStateAccountDate(planBase.getPlanNo(), null);
        if (unfinishClearingLogDate != null
            && DateTimeUtil.calcDayDiff(reportDateField.getAccountDate(), unfinishClearingLogDate) >= 0) {
            rptResultPair.setRight(MessageUtil.formatMsg("[ReportJob]-[{0}]计划在[{1}]的清算日志未汇总完成，最小日志会计日期为[{2}]，不执行报表生成",
                    planBase.getPlanNo(), reportDateField.getAccountDate(), unfinishClearingLogDate));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
    
        if (planBase.getPlanType() == PlanTypeEnum.SPV) {
            //获取受让计划
            List<String> planNosBySpvNo =Lists.newArrayList();
            Map<String, List<PlanBase>> planBySpvs = planManager.getPlanBySpvs(Lists.newArrayList(planBase.getPlanNo()));
            if(MapUtils.isNotEmpty(planBySpvs)){
                List<PlanBase> planBases = planBySpvs.get(planBase.getPlanNo());
                for(PlanBase pb : planBases){
                    if(DateTimeUtil.calcDayDiff(reportDateField.getAccountDate(),
                            DateTimeUtil.getStandardDate(pb.getEffectDate()))>=0){
                        planNosBySpvNo.add(pb.getPlanNo());
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(planNosBySpvNo)) {
                planNos.addAll(planNosBySpvNo);
                if (!planReportRepo.allFinished(planNos, reportDateField.desc(), ReportTypeEnum.RPT001)) {
                    rptResultPair.setRight(MessageUtil.formatMsg("[ReportJob]-[{0}]计划下受让计划[{1}]在[{2}]的日报未完成",
                            planBase.getPlanNo(), planNos, orgDate));
                    LogUtil.info(LOGGER, rptResultPair.getRight());
                    rptResultPair.setLeft(false);
                    return rptResultPair;
                }
            }
        }
        return rptResultPair;
    }
    
    /**
     * 月报校验
     * @param accountDate
     * @param plan
     * @param latestReport
     * @param reportMonthField
     * @return
     */
    public MutablePair<Boolean, String>  checkMonthPlanReport(Date accountDate, PlanBase plan,
                                                              PlanReport latestReport,
                                                              MonthReportDateField reportMonthField) {
        MutablePair<Boolean, String> rptResultPair = new MutablePair<>(true, "");
        /**
         * 计算当前应该出哪个月的报表： reportMonth
         */
        Date nextMonthCk = DateTimeUtil.addDays(accountDate, 1);
    
        Date reportMonth;
        if (DateTimeUtil.calMonthDiff(accountDate, nextMonthCk) > 0) {
            // 当前会计日期是每个月最后一天
            reportMonth = accountDate;
        } else {
            Date monthFirstDay = DateTimeUtil.parseFromYMD(
                    DateTimeUtil.format(accountDate, DateTimeUtil.FORMAT_DATE_YYYYMM + "01"));
            reportMonth = DateTimeUtil.addDays(monthFirstDay, -1);
        }
    
        /**
         * 根据历史已出报表（可能没出）计算应该出哪个月，从已出那个月递增：reportDateField
         */
        if (null != latestReport) {
            String reportMonthText = DateTimeUtil.format(reportMonth, DateTimeUtil.FORMAT_DATE_YYYYMM);
            reportMonthField.parse(reportMonthText);
        
            MonthReportDateField lastReportDateField = (MonthReportDateField) latestReport.getRptDate();
            if (!lastReportDateField.isEarlier(reportMonthField)) {
                rptResultPair.setRight(MessageUtil.formatMsg("[buildMonthPlanReport]-[{0}]计划在[{1}]的月管理报表已经生成",
                        plan.getPlanNo(), reportMonthField.desc()));
                LogUtil.info(LOGGER, rptResultPair.getRight());
                rptResultPair.setLeft(false);
                return rptResultPair;
            }
            reportMonthField.parse(lastReportDateField.toNext().desc());
        } else {
            // 从未有月报，从计划生效日开始处理
            reportMonthField.parse(DateTimeUtil.format(plan.getBeginDate(), DateTimeUtil.FORMAT_DATE_YYYYMM));
        }
    
        // 判断月报最后一天（月末）是否大于当前会计日期，如果是，则不生成报表（还未到月末）
        if (DateTimeUtil.calcDayDiff(reportMonthField.getAccountDate(), accountDate) > 0) {
            rptResultPair.setRight(MessageUtil.formatMsg("[buildMonthPlanReport]-[{0}]计划月报日期[{1}]大于当前会计日期[{2}]，不生成计划报表",
                    plan.getPlanNo(), reportMonthField, accountDate));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            // 不是每月最后一天，不生成报表
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
        /**
         * ------- 日期判断结束，开始生成报表 -------
         */
        // 检查上日日报是否汇总完成
        DailyReportDateField lastDailyReportDate = new DailyReportDateField();
        lastDailyReportDate.setRptDate(reportMonthField.getAccountDate());
        PlanReport dailyReport = planReportRepo.getReport(
                plan.getPlanNo(), ReportTypeEnum.RPT001, lastDailyReportDate.desc());
        if (dailyReport == null) {
            rptResultPair.setRight(MessageUtil.formatMsg("[buildMonthPlanReport]-[{0}]计划在[{1}]的日报任务未完成",
                    plan.getPlanNo(), lastDailyReportDate.desc()));
            LogUtil.info(LOGGER, rptResultPair.getRight());
            rptResultPair.setLeft(false);
            return rptResultPair;
        }
        
        return rptResultPair;
    }
}
