/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.model.domain.plan.PlanTaskLog;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.plan.PlanTaskLogRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.general.system.common.model.IEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * 每日余额校验
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.biz.shared.scheduler.job.keeper.node: BalanceCheckKeeper.java, v 0.1 2019-10-08 11:01 AM xiachenxiang Exp $
 */
@Component
public class BalanceCheckKeeper extends AbstractKeeperNode {

    @Autowired
    private PlanTaskLogRepo     planTaskLogRepo;
    @Autowired
    private PlanRepo            planRepo;
    @Autowired
    private AcctManager         acctManager;
    @Autowired
    private WkPayManager        wkPayManager;
    @Autowired
    private PlanAstManager      planAstManager;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    @Value("${wkabs.zhaolian.balancecheck.deadline:8 0 0}")
    private String deadline;

    @Override
    protected Void doHandle(Date currentTime) {
        LogUtil.info(LOGGER, "每日余额校验-start");
        currentTime = getCurrentStanderDate();
        // 判断当前时间是否大于凌晨1点
        Date deadlineTime = getDeadLineTime(currentTime, deadline);
        if (currentTime.compareTo(deadlineTime) > 0) {
            Date bizDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
            List<PlanTaskLog> tasks = planTaskLogRepo.getTasks(bizDate, PlanTaskTypeEnum.BAL_CHECKER);
            // 查询存续中的放款池计划
            List<PlanBase> onGoingPlans = planRepo.getPlansByStatus(PlanTypeEnum.SPV, PlanStatusEnum.ONGOING);
            StringBuffer sb = new StringBuffer();
            for (PlanBase plan : onGoingPlans) {
                boolean isExist = false;
                boolean isStop = false;
                if (CollectionUtils.isNotEmpty(tasks)) {
                    isExist = tasks.stream().filter(item -> item.getPlanNo().equals(plan.getPlanNo()) && item.getTaskType() == PlanTaskTypeEnum.BAL_CHECKER).findFirst().isPresent();
                }

                if (plan instanceof OrganizationEntity) {
                    OrganizationEntity organizationEntity = (OrganizationEntity) plan;
                    isStop = DateTimeUtil.calcDayDiff(orgAcctDayComponent.getTrfAcctDay().getStandardDate(), organizationEntity.getStopPayDate()) > 0;
                    if (isStop) {
                    LogUtil.info(LOGGER, "计划编号{0},当前会计日{1}大于停止放款日期{2},{3}",
                            plan.getPlanNo(), currentTime, organizationEntity.getStopPayDate(), isStop);
                    }

                }
                if (!isExist && !isStop) {
                    // 获取计划对应资金账户 平台虚拟户
                    List<Account> accounts = acctManager.getAcctByPlanNoAndType(plan.getPlanNo(), AccountTypeEnum.PLAT_VIRTUAL, false);
                    // 过滤出对应查询余额的账户
                    Optional<Account> platAcctOpt = accounts.stream()
                            .filter(item -> item.getActAttr() == AccountAttributeEnum.RECV)
                            .findFirst();
                    // 异常打印后继续
                    if (!platAcctOpt.isPresent()) {
                        LogUtil.error(LOGGER, "{0}[{1}]找不到[{2}]对应平台虚拟户", plan.getPlanType().getDesc(), plan.getPlanNo(), AccountAttributeEnum.RECV.getDesc());
                        continue;
                    }
                    Account planAcct = platAcctOpt.get();
                    Money bankBal = new Money();
                    try {
                        // 调用外部接口查询账户余额
                        WkChannelNoEnum channelNo = WkChannelNoEnum.getPlatChannelNo(planAcct.getChannelNo());
                        bankBal.addTo(wkPayManager.balance(planAcct.getActNo(), channelNo));
                        LogUtil.info(LOGGER, "{0}[{1}]查询账户余额: {2}", plan.getPlanType().getDesc(), plan.getPlanNo(), bankBal.getAmount());
                    } catch (Throwable e) {
                        LogUtil.error(LOGGER, e, "[{0}]-ChannelNo[{1}]-ActNo[{2}]调用五矿查询账户余额异常",
                                plan.getPlanNo(), planAcct.getChannelNo(), planAcct.getActNo());
                        continue;
                    }
                    if (bankBal.compareTo(new Money()) > 0) {
                        PlanAst planAst = planAstManager.getByPlanNo(plan.getPlanNo());
                        if (bankBal.subtract(planAst.getFrozenCash()).isGreaterThanZero()) {
                            String msg = MessageUtil.formatMsg("截止到{0}为止,{1}[{2}]查询账户余额为 {3},请排查具体原因 \n", DateTimeUtil.formatYMDHMSSLong(currentTime), plan.getPlanType().getDesc(), plan.getPlanNo(), bankBal.getAmount());
                            sb.append(msg);
                        }
                    }
                    PlanTaskLog planTaskLog = new PlanTaskLog();
                    planTaskLog.setExecDate(bizDate);
                    planTaskLog.setPlanNo(plan.getPlanNo());
                    planTaskLog.setTaskType(PlanTaskTypeEnum.BAL_CHECKER);
                    planTaskLogRepo.createTask(planTaskLog);
                }
            }
            if (sb.length() > 0) {
                // 获取用户
                Set<String> receivers = getAdminReceivers();
                sendSystemNotice("每日余额校验异常告警", "ALL", sb.toString(), receivers);
            }

        }
        LogUtil.info(LOGGER, "每日余额校验-end");

        return null;
    }

    @Override
    protected String getWatchJob() {
        return null;
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.BalanceCheck;
    }
}
