/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import com.abssqr.plat.biz.shared.handler.PlanAstClearingOrderHandler;
import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.general.system.common.util.LogUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.List;

/**
 * 承接池受让计划 与 放款池计划
 *  结算单进行支付交易处理
 
 三 增加清算支付处理任务 PlanAstClearingOrderHandler
 1、拉取待支付的 结算单 TaskStatusEnum.INIT
 2、根据对手机构进行汇总支付与收入，进行轧差计算，创建支付订单，一个对手机构对应一笔支付单
 PlanCapitalTransTypeEnum 交易类型增加 支付 与 收入 类型进行对应
 3、修改结算单状态为 WAITING，等待支付结果
 4、变更资产水位表中待结算 与 在途金额
 
 四 修改 PlanCapitalOrderListener 交易单结果监听器
 1、增加选择处理器，根据不同 PlanCapitalTransTypeEnum 类型，进行对应业务处理剥离 分配兑付与清算
 2、清算类型交易单结果处理：
 2.1、拉取交易单据对应交易对手的结算单，根据实际确认金额，循环每条扣减金额及更改状态为SUCCESS
 2.2、对支付金额不足剩余的 结算单 更改状态为 INIT，等待下次支付
 2.3、根据交易单据金额更改扣减水位表的 待结算支出 或 待结算收入 金额
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.handler: PlanAstStatHandler.java, v 0.1 2019-06-12 20:44 zhenxuan.luo Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class PlanAstClearingOrderJob extends BaseQuartzJob {
    @Autowired
    private PlanRepo                    planRepo;
    @Autowired
    private TransactionTemplate         transactionTemplate;
    @Autowired
    private PlanAstClearingOrderHandler planAstClearingOrderHandler;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 每天 25 2/5 * * * ?
     */
    @Value("${sync.quartz.plan.clear}")
    private String              cron;
    
    @Override
    protected String getCronExpr() {
        return cron;
    }
    
    @Override
    protected boolean isRequestRecovery() {
        return true;
    }
    
    @Override
    public void doExecute() {
        // 获取转让会计日
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        LogUtil.info(LOGGER, "[ClearingOrderJob]-[{0}]结算单进行支付交易处理任务start", curDate);
    
        // 1.1 查询当前 状态为 存续中 承接池计划
        List<PlanBase> onGoingPlans = planRepo.getPlansByStatus(null,PlanStatusEnum.ONGOING);

        LogUtil.info(LOGGER, "[ClearingOrderJob]-当前存续期计划数量[{0}]", onGoingPlans.size());
        for (PlanBase entity : onGoingPlans) {
            final String planNo = entity.getPlanNo();
            try {
                LogUtil.info(LOGGER, "{0}[{1}]结算单进行支付交易处理开始", entity.getPlanType().getDesc(), planNo);
                transactionTemplate.execute(status -> {
                    planAstClearingOrderHandler.handle(entity);
                    return null;
                });
                LogUtil.info(LOGGER, "{0}[{1}]结算单进行支付交易处理结束", entity.getPlanType().getDesc(), planNo);
                
            } catch (AbssqrBizException e) {
                if (CommonErrorCodeEnum.NO_DATA == e.getErrCode()) {
                    LogUtil.warn(LOGGER, e.getMessage());
                } else {
                    LogUtil.error(LOGGER, e, "{0}[{1}]结算单进行支付交易处理异常", entity.getPlanType().getDesc(), planNo);
                }
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "{0}[{1}]结算单进行支付交易处理异常", entity.getPlanType().getDesc(), planNo);
            }
        }
    }
}