/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanCapitalOrderDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.ThreadUtil;
import com.general.system.tool.util.ToolUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

/**
 * 计划资金支付交易单发起处理
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.scheduler.job.plan: PlanCapitalOrderJob.java, v 0.1 2019-06-18 21:19 zhenxuan
 * .luo Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class PlanCapitalOrderJob extends BaseQuartzJob {
    
    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private PlanCapitalOrderRepo    planCapitalOrderRepo;
    @Autowired
    private TransactionTemplate     transactionTemplate;
    /**
     * 每天 30 3/5 * * * ?
     */
    @Value("${sync.quartz.plan.capital}")
    private String                  cron;
    /**
     * 每天 30 3/5 * * * ?
     */
    @Value("${sync.space.capital:3000}")
    private Long                  capital;
    
    @Override
    protected String getCronExpr() {
        return cron;
    }
    
    @Override
    protected boolean isRequestRecovery() {
        return true;
    }
    
    @Override
    public void doExecute() {
        
        Date curExecTime = SystemDateUtil.getSystemDate();
        // 获取 交易单据 改为查询 所有 init 状态的支付交易单 进行执行  使用分页查询减小外部并发
        PlanCapitalOrderPage page = new PlanCapitalOrderPage();
        page.setStatus(TaskStatusEnum.INIT.getCode());
        page.setNextExecTime(curExecTime);
        AtomicLong minId = new AtomicLong(0L);

        do {
            page.setId(minId.get());
            // 1、查询资金单据
            planCapitalOrderRepo.getCapitalOrderByStatus(page);
            if (ToolUtil.isEmpty(page.getDatas())) {
                LogUtil.info(LOGGER, "[CapitalOrderJob]-[{0}]资金支付交易单没有可执行数据", curExecTime);
                return;
            }
            LogUtil.info(LOGGER, "[CapitalOrderJob]-[{0}]资金支付交易单发起处理任务start", curExecTime);
            for (PlanCapitalOrderDO order : page.getDatas()) {
                // 循环查询条件 设置最小id
                minId.set( minId.get()<order.getId() ? order.getId() : minId.get());
                try {
                    transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                        @Override
                        protected void doInTransactionWithoutResult(TransactionStatus status) {
                            // 交易单 加锁
                            PlanCapitalOrder lockOrdr = planCapitalOrderRepo.lockBySeqNo(order.getSeqNo());
                            // 执行时间判断
                            if (curExecTime.compareTo(lockOrdr.getProcTime()) >= 0) {
                                JobControl jobControl = jobControlRepository.selectByJobName(getJobName());
                                if (!jobControl.getIsEff().isYes()) {
                                    LogUtil.warn(LOGGER, "计划资金单[{0}]终止，任务控制生效标记为[{1}]", getJobName(), jobControl.getIsEff());
                                    return;
                                }
                                planCapitalOrderManager.processCapitalOrder(lockOrdr, SystemDateUtil.getSystemDate());
                            }
                        }
                    });
                } catch (AbssqrBizException e) {
                    if (CommonErrorCodeEnum.NO_DATA == e.getErrCode()) {
                        LogUtil.warn(LOGGER, e.getMessage());
                    } else {
                        LogUtil.error(LOGGER, e, "计划资金单[{0}]发起交易异常:[{1}]", order.getSeqNo(), e.getMessage());
                    }
                } catch (Throwable e) {
                    LogUtil.error(LOGGER, e, "计划资金单[{0}]发起交易异常:[{1}]", order.getSeqNo(), e.getMessage());
                }finally {
                    ThreadUtil.sleep(capital,LOGGER);
                }
            }

        } while (page.hasNextPage());
        LogUtil.info(LOGGER, "[CapitalOrderJob]-[{0}]资金支付交易单发起处理任务end:[{1}]", curExecTime, minId);
    }
}