/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanCapitalOrderDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResultItem;
import com.abssqr.plat.common.model.convertor.PlanCapitalOrderConvertor;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.abssqr.plat.core.share.pay.PlanCapitalFinishRequest;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Maps;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

/**
 * 计划资金支付交易单结果查询
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.scheduler.job.plan: PlanCapitalOrderJob.java, v 0.1 2019-06-18 21:19 zhenxuan
 * .luo Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class PlanCapitalOrderResJob extends BaseQuartzJob {

    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;

    @Autowired
    private PlanCapitalOrderRepo planCapitalOrderRepo;
    @Autowired
    private TransactionTemplate  transactionTemplate;
    @Autowired
    private WkPayManager         wkPayManager;
    /**
     * 每天 30 4/5 * * * ?
     */
    @Value("${sync.quartz.plan.query}")
    private String               cron;

    @Override
    protected String getCronExpr() {
        return cron;
    }

    @Override
    protected boolean isRequestRecovery() {
        return true;
    }

    /**
     * 支付结果查询
     * 1、查询资金单据 状态为 {@link TaskStatusEnum WAITING}
     * 2、以 planCapitalOrder.getOutAcptNo() 对应 tradeNo 查询五矿订单结果接口
     * 3、更新 资金单据 支付结果
     *
     * @throws JobExecutionException
     */
    @Override
    public void doExecute() {
        Date curExecTime = SystemDateUtil.getSystemDate();
        AtomicInteger count = new AtomicInteger();
        
        PlanCapitalOrderPage page = new PlanCapitalOrderPage();
        page.setNextExecTime(curExecTime);
        page.setStatus(TaskStatusEnum.WAITING.getCode());
        AtomicLong minId = new AtomicLong(0L);
        do {
            page.setId(minId.get());
            // 1、查询资金单据
            planCapitalOrderRepo.getCapitalOrderByStatus(page);
            if (ToolUtil.isEmpty(page.getDatas())) {
                LogUtil.info(LOGGER, "[CapitalOrderResJob]-[{0}]资金支付交易单没有可执行数据", curExecTime);
                return;
            }
            LogUtil.info(LOGGER, "[CapitalOrderResJob]-[{0}]资金支付交易单结果查询任务start", curExecTime);
            List<String> entryNos = new ArrayList<>(page.getDatas().size());
            Map<String, PlanCapitalOrder> tradeMap = page.getDatas().stream().map(one -> {
                entryNos.add(one.getOutAcptNo());
                minId.set(minId.get()<one.getId()?one.getId():minId.get());
                return one;
            }).collect(Collectors
                               .toMap(PlanCapitalOrderDO::getOutAcptNo,
                                       item -> PlanCapitalOrderConvertor.convert2Domain(item)));
            LogUtil.info(LOGGER, "[CapitalOrderResJob]-[{0}]资金支付交易单结果查询任务-minId={1}", curExecTime, minId.get());
            try {
                // 2、查询五矿订单结果接口
                TradeResult result = wkPayManager.tradeRstQry(entryNos);

                final Map<String, TradeResultItem> enNoTradeMap = Maps.newHashMap();
                if (ToolUtil.isNotEmpty(result.getPayInfoList())) {
                    enNoTradeMap.putAll(result.getPayInfoList().stream().collect(
                        Collectors.toMap(TradeResultItem::getEntryNo, tradeResultItem -> tradeResultItem)));
                }

                tradeMap.forEach((entryNo, order) -> {
                    transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                        @Override
                        protected void doInTransactionWithoutResult(TransactionStatus status) {
                            TradeResultItem item = enNoTradeMap.get(entryNo);
                            if (item == null) {
                                LogUtil.warn(LOGGER,"资金支付交易单结果查询-未查到条目号对应的结果，等待下次查询:{0}", entryNo);
                            } else {
                                PlanCapitalFinishRequest finishRequest = new PlanCapitalFinishRequest();
                                finishRequest.setSeqNo(order.getSeqNo());
                                finishRequest.setOutTransNo(item.getEntryNo());
                                finishRequest.setCfrmAmt(Money.createWithCent(item.getPayMoney()));
                                boolean isOver = false;
                                // 判断查询结果状态
                                switch (item.getStatus()) {
                                    case 成功:
                                        finishRequest.setRes(TaskStatusEnum.SUCCESS);
                                        finishRequest.setMemo(item.getStatus().getDesc());
                                        isOver = true;
                                        break;
                                    case 处理中:
                                        // 处理中不做操作继续查询
                                        break;
                                    case 支付平台异常:
                                        //什么都不做，发告警 fixme 发站内信，运营人员通知支付平台排查问题。
                                        LogUtil.error(LOGGER,"支付平台异常-[{0}]",entryNo);
                                        break;
                                    default:
                                        finishRequest.setRes(TaskStatusEnum.FAILURE);
                                        finishRequest.setMemo(item.getStatus().getDesc());
                                        isOver = true;
                                }
                                if (isOver) {
                                    // 3、更新 资金单据 支付结果
                                    planCapitalOrderManager.finishCapitalOrder(finishRequest);

                                }
                            }
                            count.getAndIncrement();
                        }
                    });

                });


            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "查询五矿订单结果接口异常:{0}", e.getMessage());
            }
        } while (page.hasNextPage());
        LogUtil.info(LOGGER, "[CapitalOrderResJob]-[{0}]资金支付交易单结果查询任务end,exec[{1}]", curExecTime, count.get());
    }
}