/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk.domain.config;

import cn.hutool.core.io.FastByteArrayOutputStream;
import cn.hutool.core.io.IoUtil;
import com.abssqr.gateway.util.SftpUtil;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.utils.SftpFileUtil;
import com.jcraft.jsch.ChannelSftp;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.io.InputStream;

/**
 * sftp 配置
 * @author hanfei
 * @version $Id: WkSftpConfig.java, v 0.1 2019-09-19 4:23 PM hanfei Exp $
 */
@Component
@ConfigurationProperties(prefix = "wkabs.upload")
public class WkSftpConfig {
    
    private static ThreadLocal<SftpFileUtil> selfChannelSftpThreadLocal = new ThreadLocal<>();
    
    /**
     * sftp 下载
     */
    public FastByteArrayOutputStream downloadSftp(String fileSrc){
        try {
            InputStream stream = SftpUtil.getInputStream(fileSrc, this.selfChannelSftp());
            return IoUtil.read(stream);
        } catch (Exception e) {
            throw new AbssqrBizException("下载文件异常", CommonErrorCodeEnum.SFTP_ERROR, e);
        } finally {
            close();
        }
    }
    
    /**
     * sftp 上传
     */
    public boolean uploadSftp(InputStream stream, String fileName, String filePath){
        try {
            boolean result = SftpUtil.uploadFile(filePath, fileName, stream, this.selfChannelSftp());
            if (!result){
                throw new AbssqrBizException("上传文件失败", CommonErrorCodeEnum.SFTP_ERROR);
            }
            return result;
        } catch (Exception e) {
            throw new AbssqrBizException("上传文件异常", CommonErrorCodeEnum.SFTP_ERROR, e);
        } finally {
            close();
        }
    }
    
    /**
     * 获取sftp 连接通道
     * @return
     */
    public ChannelSftp selfChannelSftp() {
        if (selfChannelSftpThreadLocal.get() == null) {
            SftpFileUtil _sftpFileUtil = new SftpFileUtil(
                    this.getHost(),
                    this.getPort(),
                    this.getUsername(),
                    this.getPassword());
            selfChannelSftpThreadLocal.set(_sftpFileUtil);
        }
        return selfChannelSftpThreadLocal.get().getChannel();
    }
    
    /**
     * 关闭连接通道
     */
    public static void close() {
        if (selfChannelSftpThreadLocal.get() != null) {
            selfChannelSftpThreadLocal.get().close();
            selfChannelSftpThreadLocal.set(null);
        }
    }
    
    /**
     * 上传方式
     */
    private String type;
    /**
     * 地址
     */
    private String host;
    /**
     * 端口
     */
    private Integer port;
    /**
     * 用户名
     */
    private String username;
    /**
     * 密码
     */
    private String password;
    /**
     * 超时时间
     */
    private String timeout;
    /**
     * 上传路径
     */
    private String path;
    
    public String getHost() {
        return host;
    }
    
    public void setHost(String host) {
        this.host = host;
    }
    
    public Integer getPort() {
        return port;
    }
    
    public void setPort(Integer port) {
        this.port = port;
    }
    
    public String getUsername() {
        return username;
    }
    
    public void setUsername(String username) {
        this.username = username;
    }
    
    public String getPassword() {
        return password;
    }
    
    public void setPassword(String password) {
        this.password = password;
    }
    
    public String getTimeout() {
        return timeout;
    }
    
    public void setTimeout(String timeout) {
        this.timeout = timeout;
    }
    
    public String getType() {
        return type;
    }
    
    public void setType(String type) {
        this.type = type;
    }
    
    public String getPath() {
        return path;
    }
    
    public void setPath(String path) {
        this.path = path;
    }
}
