/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.share.pay;

import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanClearingOrderItem;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransAcct;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalLog;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.model.convertor.PlanCapitalLogConvertor;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.ext.AstSyncTaskContext;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalLogRepo;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.abssqr.plat.core.share.process.PlanCapitalFinishChangeStage;
import com.general.system.common.model.Money;
import com.general.system.common.util.*;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 计划资金订单操作
 *
 * @author zhenxuan.luo
 * @version PlanCapitalOrderManager.java, v 0.1 2019-06-03 16:43 zhenxuan.luo Exp $
 */
@Component
public class PlanCapitalOrderManager {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanCapitalOrderManager.class);
    
    @Autowired
    private PlanCapitalOrderRepo            planCapitalOrderRepo;
    @Autowired
    private PlanCapitalLogRepo              planCapitalLogRepo;
    @Autowired
    private PlanCapitalFinishProcessFactory planCapitalFinishProcessFactory;
    @Autowired
    private PlanCapitalFinishChangeStage    planCapitalFinishChangeStage;
    @Autowired
    private AcctManager                     acctManager;
    @Autowired
    private WkPayManager                    wkPayManager;
    @Autowired
    private PlanClearingOrderRepo           planClearingOrderRepo;
    @Autowired
    private PlanManager                     planManager;
    @Autowired
    private AstSyncTaskCtrRepo              astSyncTaskCtrRepo;

    /**
     * 初始化资金订单
     * 清算、分配统一支付交易
     * @return
     */
    public PlanCapitalOrder createCapitalOrder(PlanCapitalCreateRequest request) {
        String seqNo = SequenceUtil.genSeqNo(request.getTransType().getCode());
        PlanCapitalOrder newOrder = new PlanCapitalOrder();
        newOrder.setPlanNo(request.getPlanNo());
        newOrder.setBizNo(request.getBizNo());
        newOrder.setSeqNo(seqNo);
        newOrder.setTransAmt(request.getTransAmt());
        newOrder.setPayableAmt(request.getPayableAmt());
        newOrder.setAccountDate(request.getAccountDate());
        // 交易时间
        newOrder.setTransTime(SystemDateUtil.getSystemDate());
        newOrder.setTransType(request.getTransType());
        newOrder.setCntPtyCode(request.getTargetOrgCode());
        // 差异金额
        Money diffAmt = newOrder.getPayableAmt().subtract(newOrder.getTransAmt());
        newOrder.setMemo(MessageUtil.formatMsg("[{0}]-[{1}]-计划[{2}]对手[{3}]: {4}-应付[{5}]-交易[{6}]-差异[{7}]",
                seqNo, request.getAccountDate(), request.getPlanNo(), request.getTargetOrgCode(), request.getTransType().getCode(),
                newOrder.getPayableAmt(), newOrder.getTransAmt(), diffAmt));
        // 追加备注内容
        if (ToolUtil.isNotEmpty(request.getMemo())){
            newOrder.setMemo(newOrder.getMemo().concat(request.getMemo()));
        }
        newOrder.setStatus(TaskStatusEnum.INIT);
        // 可执行时间
        newOrder.setProcTime(SystemDateUtil.getSystemDate());
        // 下次可执行时间
        newOrder.setNextExecTime(SystemDateUtil.getSystemDate());
        
        // 实际交易金额大于0 创建可交易支付单
        if (newOrder.getTransAmt().isGreaterThanZero()){
            planCapitalOrderRepo.insert(newOrder);
        }else{
            // 实际交易金额小于等于0 时 创建备份支付单 状态记忽略
            newOrder.setStatus(TaskStatusEnum.IGNORE);
            planCapitalOrderRepo.insertHis(newOrder);
        }
        return newOrder;
    }
    
    /**
     * 处理资金订单
     * @param planCapitalOrder
     * @return
     */
    public void processCapitalOrder(PlanCapitalOrder planCapitalOrder, Date curTime) {
        VarChecker.checkArgument(!planCapitalOrder.getStatus().isFinish(), "计划[{0}]的支付单[{1}]已结束，不再进行处理",
                planCapitalOrder.getPlanNo(), planCapitalOrder.getSeqNo());
        
        if (planCapitalOrder.getStatus() != TaskStatusEnum.INIT) {
            LogUtil.warn(LOGGER, "支付单状态非初始化，已经发起支付请求，不再重复处理 - [{0}]", planCapitalOrder.getSeqNo());
            return;
        }
        
        // 填入账号
        this.fillAccount(planCapitalOrder);
        // 发起平台户
        Account srcAcct = this.getAcctByType(planCapitalOrder, AccountTypeEnum.PLAT_VIRTUAL);
        // 目的银行户
        Account tgtAcct = this.getAcctByType(planCapitalOrder, AccountTypeEnum.BANK);
        // 获取备注
        String remark1 = this.getRemark(planCapitalOrder);
        // 在线交易
        String enNo = wkPayManager.trade(srcAcct, tgtAcct, planCapitalOrder.getTransAmt(),remark1);
        planCapitalOrder.setOutAcptNo(enNo);
        
        planCapitalOrder.setStatus(TaskStatusEnum.WAITING);
        //发起交易系统时间
        planCapitalOrder.setTransTime(SystemDateUtil.getSystemDate());
        // 更新下一次执行时间增加5分钟后
        planCapitalOrder.setNextExecTime(DateTimeUtil.addMinutes(curTime, AbsConfig.timeStampBuffer));

        int refCnt = planCapitalOrderRepo.updateProcess(planCapitalOrder);
        LogUtil.info(LOGGER, "支付单[{0}]受理成功[{1}]，执行状态更新结果[{2}]", planCapitalOrder.getSeqNo(), enNo, refCnt);
        
    }

    /**
     * 获取备注信息
     * @param planCapitalOrder
     * @return
     */
    private String getRemark(PlanCapitalOrder planCapitalOrder) {
        String result = null;
        String planNo = planCapitalOrder.getPlanNo();
        PlanBase planBase = planManager.getPlanFullByNo(planNo);
        if (planBase.getPlanType() == PlanTypeEnum.SPV){
            String managerOrgCode = ((OrganizationEntity) planBase).getManagerOrgCode();
            AstSyncTaskCtr astSyncTaskCtr = astSyncTaskCtrRepo.getAstSyncTaskCtr(managerOrgCode);
            if (astSyncTaskCtr != null) {
                AstSyncTaskContext extData = astSyncTaskCtr.getExtData();
                Map<String, Map<String, String>> capitalOrderRemarkMap = extData.getCapitalOrderRemarkMap();
                result = Optional.ofNullable(capitalOrderRemarkMap.get(planNo)).map(map -> map.get(planCapitalOrder.getTransType().getCode())).orElse(null);
            }
        }
        return result;
    }

    /**
     * 处理完成（成功或者失败）
     * @param planCapitalFinishRequest
     * @return
     */
    public PlanCapitalOrder finishCapitalOrder(PlanCapitalFinishRequest planCapitalFinishRequest) {
        PlanCapitalOrder planCapitalOrder = planCapitalOrderRepo.lockBySeqNo(planCapitalFinishRequest.getSeqNo());
        VarChecker.checkNotNull(planCapitalOrder, "支付单[{0}]不存在，或已经被处理", planCapitalFinishRequest);
        planCapitalOrder.setOutTransNo(planCapitalFinishRequest.getOutTransNo());
        // 完成系统时间
        planCapitalOrder.setFinishTime(SystemDateUtil.getSystemDate());
        planCapitalOrder.setCfrmAmt(planCapitalFinishRequest.getCfrmAmt());
        planCapitalOrder.setStatus(planCapitalFinishRequest.getRes());
        planCapitalOrder.setMemo(
                MessageUtil.formatMsg("{0}-{1}", planCapitalOrder.getMemo(), planCapitalFinishRequest.getMemo()));
        // 1、插入历史表
        planCapitalOrderRepo.insertHis(planCapitalOrder);
        // 如果成功，进一步处理
        if (planCapitalOrder.getStatus() == TaskStatusEnum.SUCCESS) {
            // 2、删除支付单据
            planCapitalOrderRepo.deleteBySeqno(planCapitalOrder.getSeqNo(), planCapitalOrder.getPlanNo());
            PlanCapitalLog planCapitalLog = PlanCapitalLogConvertor.buildPlanCapitalLog(planCapitalOrder);
            planCapitalLogRepo.insert(planCapitalLog);
            
            // 3、更新水位金额变化及登记流水
            planCapitalFinishChangeStage.changeAstByCapitalOrder(planCapitalOrder);
            
            // 4、针对不同交易场景 后续 特殊处理业务
            planCapitalFinishProcessFactory.process(planCapitalLog);
        } else {
            //还原代理交易，重新支付
            Date nextExecTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 5);
            planCapitalOrderRepo.restoreCapitalOrder(nextExecTime, planCapitalFinishRequest.getSeqNo());
        }
        return planCapitalOrder;
    }
    
    /**
     * 填充账号信息
     * @param planCapitalOrder
     * return 对手机构的放款户
     */
    private void fillAccount(PlanCapitalOrder planCapitalOrder) {
        // 查询计划 所有账户
        List<Account> planAccounts = acctManager.getAllAcctByPlanNo(planCapitalOrder.getPlanNo(),
                IEnumUtil.getEnumList(AccountTypeEnum.class));
        // 源账户
        Account planAcct = this.filterByAttr(planCapitalOrder.getTransType().getSrcAcct(),
                planAccounts);
        VarChecker.checkNotNull(planAcct, "资金操作[{0}]计划账户不能为空", planCapitalOrder);
        
        // 对手机构所有账户
        List<Account> cntAccounts = acctManager.getAllAcctByPlanNo(planCapitalOrder.getCntPtyCode(),
                IEnumUtil.getEnumList(AccountTypeEnum.class));
        // 目的账户
        Account cntPtyAcct = this.filterByAttr(planCapitalOrder.getTransType().getDesAcct(),
                cntAccounts);
        
        VarChecker.checkNotNull(cntPtyAcct, "资金操作[{0}]找不到合适的对手方账号", planCapitalOrder);
        
        planCapitalOrder.setPlanAcct(planAcct);
        
        planCapitalOrder.setCntPtyAcct(cntPtyAcct);
        
    }
    
    private Account filterByAttr(PlanCapitalTransAcct transAcct, List<Account> accountList) {
        Account target = null;
        for (Account account : accountList) {
            if (account.getActAttr() == transAcct.getAttribute()
                && account.getActType() == transAcct.getAccountType()) {
                target = account;
                break;
            }
        }
        return target;
    }
    
    /**
     * 按账户类型找账户
     * @param capitalOrder
     * @return
     */
    private Account getAcctByType(PlanCapitalOrder capitalOrder, AccountTypeEnum type) {
        if (capitalOrder.getPlanAcct().getActType() == type) {
            return capitalOrder.getPlanAcct();
        }
        if (capitalOrder.getCntPtyAcct().getActType() == type) {
            return capitalOrder.getCntPtyAcct();
        }
        throw new AbssqrBizException(MessageUtil.formatMsg("计划[{0}]交易单seqNo[{1}]没有匹配[{2}]的账户",
                capitalOrder.getPlanNo(), capitalOrder.getSeqNo(), type.getCode()));
    }
    
    /**
     * 查询计划当前未完成结算单及交易支付单的数量
     * 报表前置条件
     * @param planNo
     * @return
     */
    public int reportCheckCapitals(String planNo, Day orgDate) {
        int count = 0;
        // 结算单
        List<PlanClearingOrderItem> clearOrder =
                planClearingOrderRepo.queryByClearingOrderStat(planNo, TaskStatusEnum.INIT, orgDate.getStandardDate());
        if (ToolUtil.isNotEmpty(clearOrder)){
            count += clearOrder.size();
        }
        
        // 判断机构会计日不分类型支付单数量
        count += planCapitalOrderRepo.getActiveCntByCondition(orgDate.getStandardDate(), planNo, null,
                Lists.newArrayList(TaskStatusEnum.INIT, TaskStatusEnum.WAITING, TaskStatusEnum.IGNORE)).intValue();
        return count;
    }
    
    
    /**
     * 验证当前是否存在支付单
     * 账户是否可以进行余额查询
     * @return
     */
    public boolean checkExistCapital(String planNo) {
        // 查询当前正在进行中的支付单
        Long cnt = planCapitalOrderRepo.getActiveCntByCondition(null, planNo, null,
                Lists.newArrayList(TaskStatusEnum.INIT, TaskStatusEnum.WAITING));
        // 判断 当前已有支付单进行则不进行余额查询
        if (cnt > 0) {
            LogUtil.info(LOGGER, "查询到[{0}条]支付任务" , cnt);
            return true;
        }
        return false;
    }
    
    
    /**
     * 验证当前是否存在支付单结算单
     * 停止放款日放款户转还款户
     * @return
     */
    public boolean repayAcctCapital(String planNo) {
        int count = 0;
        // 查询所有结算单
        List<PlanClearingOrderItem> clearOrder =
                planClearingOrderRepo.queryByClearingOrderStat(planNo, TaskStatusEnum.INIT, null);
        if (ToolUtil.isNotEmpty(clearOrder)){
            count += clearOrder.size();
        }
        // 查询当前所有支付单
        count += planCapitalOrderRepo.getActiveCntByCondition(null, planNo, null, null);
        // 判断 当前已有支付单进行则不进行余额查询
        if (count > 0) {
            LogUtil.info(LOGGER, "查询到[{0}条]待结算或支付任务" , count);
            return true;
        }
        return false;
    }

}